package com.asreader.p3xu.view;

import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUMemoryBank;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;

import java.lang.ref.WeakReference;
import java.util.Locale;

import com.asreader.R;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.view.base.MemoryActivity;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

public class WriteMemoryActivity extends MemoryActivity {

	private static final String TAG = WriteMemoryActivity.class.getSimpleName();

	private TextView txtValue;

	private Button btnWrite;
	private String mValue;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_write_memory);
		ActionBar actionBar = getActionBar();
		if (actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}
		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.write_memory_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createAsReaderP3xU();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initAsReaderP3xU();

		InitTask task = new InitTask(this);
		task.execute();
		
//		// Initialize Mask
//		initMask();
//
//		resetActionResult();
//
//		setMask(getMask());
//
//		// Enable action widget
//		enableActionWidgets(true);
//
//		outputMessage(R.string.write_memory_intro_message);
	}

	@Override
	protected void onDestroy() {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyAsReaderP3xU();

		super.onDestroy();
	}

	@Override
	protected void onStart() {
		super.onStart();
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
	}

	@Override
	protected void onStop() {
		
		if(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.WriteMemory)
			getAsReaderP3xU().stop();

		// Rollback Mask
		exitMask();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStop()");
		super.onStop();
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onResume()");
	}

	@Override
	protected void onPause() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onPause()");
		super.onPause();
	}
	
	@Override
	public void onClick(View v) {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));
		
		switch (v.getId()) {
		case R.id.write_value:
			showWriteValueDialog();
			break;
		case R.id.action_write:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				writeMemory();
			} else {
				stopAction();
			}
			break;
		}
		super.onClick(v);
	}

	@Override
	public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {
		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "EVENT. onActionChanged(%s)", action);

		enableActionWidgets(true);

		if (action == AsReaderP3xUActionState.Stop) {
			if (!isActionResult()) {
				outputMessage(R.string.write_memory_intro_message);
			}
		}
	}

	@Override
	public void onAccessResult(AsReaderP3xU asReaderP3xU, AsReaderP3xUResultCode code, AsReaderP3xUActionState action, String epc, String data) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s])", code, action, epc == null ? "" : epc,
				data == null ? "" : data);

		setActionResult();

		if (code != AsReaderP3xUResultCode.NoError) {
			outputFailMessage(code.toString());
			return;
		}

		if(epc != null)
			outputSelection(epc.length() > 4 ? epc.substring(4) : epc);

		outputSuccessMessage(R.string.access_success);
	}

	@Override
	protected void clearWidgets() {
		super.clearWidgets();
		
		resetActionResult();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. clearWidgets()");
	}

	// Write Memory
	private void writeMemory() {
		String data = getWriteValue();

		if (data.length() % 4 != 0 || data.length() == 0) {
			AlertDialog.Builder builder = new AlertDialog.Builder(WriteMemoryActivity.this);
			builder.setTitle("Error");
			builder.setMessage("Write Data must be entered in WORD unit.(1WORD = 16-bit)")
					.setCancelable(false)
					.setPositiveButton("OK", null);

			AlertDialog dlg = builder.create();
			dlg.show();
		} else if (data.length() / 4 > 32) {
			AlertDialog.Builder builder = new AlertDialog.Builder(WriteMemoryActivity.this);
			builder.setTitle("Error");
			builder.setMessage("Write Data cannot be exceed 32WORD.(1WORD = 16-bit)")
					.setCancelable(false)
					.setPositiveButton("OK", null);

			AlertDialog dlg = builder.create();
			dlg.show();
		}else  {
			clearWidgets();
			enableActionWidgets(false);
			outputSelection("");
			outputMessage(R.string.read_memory_read_message);

			resetActionResult();

			AsReaderP3xUMemoryBank bank = getMemBank();
			int offset = getMemOffset();

			int time = getOperationTime();

			String password = getAccessPassword();
			try {
				Thread.sleep(50);
				getAsReaderP3xU().setAccessPassword(password);
			} catch (AsReaderP3xUException | InterruptedException e) {
				AsReaderP3xUASRLog.e(TAG, "ERROR. writeMemory() - Failed to set password [%s]", password);
			}

			try {
				Thread.sleep(50);
				getAsReaderP3xU().setOperationTime(time);
			} catch (AsReaderP3xUException | InterruptedException e) {
				AsReaderP3xUASRLog.e(TAG, "ERROR. writeMemory() - Failed to set operation time(%d)", time);
			}

			AsReaderP3xUResultCode res;
			if ((res = getAsReaderP3xU().writeMemory(bank, offset, data)) != AsReaderP3xUResultCode.NoError) {
				AsReaderP3xUASRLog.e(TAG, "ERROR. writeMemory() - Failed to start write memory(%s, %d, [%s])", bank, offset, data);
				String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
				Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
				enableActionWidgets(true);
				outputFailMessage(R.string.write_memory_fail_write_message);
				return;
			}
			AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. writeMemory()");
		}
	}

	// Initialize Reader
	@Override
	protected void initAsReaderP3xU() {
		super.initAsReaderP3xU();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initAsReaderP3xU()");
	}

	// Initialize Widgets
	@Override
	protected void initWidgets() {
		super.initWidgets();

		ScrollView scrlParams = (ScrollView) findViewById(R.id.write_params);
		scrlParams.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));
		
		// Value EditText
		txtValue = (TextView) findViewById(R.id.write_value);
		txtValue.setOnClickListener(this);
		setWriteValue("");
		// Write Button
		btnWrite = (Button) findViewById(R.id.action_write);
		btnWrite.setOnClickListener(this);
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		btnWrite.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.WriteMemory));
		if (enabled) {
			switch (getAsReaderP3xU().getAction()) {
			case WriteMemory:
				btnWrite.setText(R.string.action_stop);
				break;
			case Stop:
				btnWrite.setText(R.string.action_write_memory);
				break;
			default:
				break;
			}
		}
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}
	
	protected void setWriteValue(String value) {
		mValue = value;
		txtValue.setText(mValue);
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setWriteValue([%s])", value);
	}
	
	protected String getWriteValue() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. getWriteValue() - [%s]", mValue);
		return mValue;
	}
	
	private void showWriteValueDialog() {
		if (!mIsEnabled)
			return;
		
		LinearLayout root = (LinearLayout)LinearLayout.inflate(this, R.layout.dialog_input_hex, null);
		final EditText writeValue = (EditText)root.findViewById(R.id.value);
		
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.write_data);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				setWriteValue(writeValue.getText().toString());
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(writeValue.getWindowToken(), 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showWriteValueDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(writeValue.getWindowToken(), 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showWriteValueDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
			
			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(writeValue.getWindowToken(), 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showWriteValueDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {
			
			@Override
			public void onShow(DialogInterface dialog) {
				writeValue.setText(mValue);
				writeValue.selectAll();
				writeValue.requestFocus();
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.showSoftInput(writeValue, InputMethodManager.SHOW_FORCED);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showWriteValueDialog().onShow()");
			}
		});
		dialog.show();
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showWriteValueDialog()");
	}
	
	private static class InitTask extends AsyncTask<Void, Void, Boolean> {
		private final WeakReference<WriteMemoryActivity> mActivity;

		InitTask(WriteMemoryActivity activity) {
			mActivity = new WeakReference<>(activity);
		}

		@Override
		protected void onPreExecute() {
			WriteMemoryActivity activity = mActivity.get();
			WaitDialog.show(activity, R.string.init_mask);
			super.onPreExecute();
		}

		@Override
		protected Boolean doInBackground(Void... params) {
			// Initialize Mask
			WriteMemoryActivity activity = mActivity.get();
			activity.initMask();
			
			return true;
		}
		
		@Override
		protected void onPostExecute(Boolean result) {
			WriteMemoryActivity activity  = mActivity.get();
			activity.resetActionResult();

			activity.setMask(activity.getMask());

			// Enable action widget
			activity.enableActionWidgets(true);

			activity.outputMessage(R.string.write_memory_intro_message);
			WaitDialog.hide();
			super.onPostExecute(result);
		}
	}
}
