package com.asreader.support.andro.app.rfid.util;

import android.content.Context;
import android.content.SharedPreferences;

import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.rfid.params.AsReaderP3xUSelectMaskEpcParam;
import com.asreader.p3xu.rfid.type.AsReaderP3xUBuzzerState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUQuerySession;
import com.asreader.p3xu.rfid.type.AsReaderP3xUSessionFlag;
import com.asreader.p3xu.rfid.type.AsReaderP3xUVibratorState;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.util.ArrayList;

/**
 * Created by Kastro on 2018-07-25.
 */

public class UserConfigs {

    private static final String TAG = UserConfigs.class.getSimpleName();
    private static ConfigParams _configParams = null;
    private final String KEY = "mask_params";

    private Context _context = null;
    private Gson _gson = null;
    private static UserConfigs sInstance;


    public static UserConfigs getInstance(Context context) {
        if(sInstance == null){
            sInstance = new UserConfigs(context);
        }
        return sInstance;
    }


    public void setContext(Context context) {
        if (_context == null) _context = context;
        if (_gson == null) _gson = new GsonBuilder().create();
    }

    public UserConfigs(Context context) {
        _context = context;
        _gson = new GsonBuilder().create();
    }

    public void Save() {
        String maskParams = _gson.toJson(_configParams, ConfigParams.class);
        SharedPreferences sharedPreferences = _context.getSharedPreferences(com.asreader.p3xu.MainActivity.APP_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString(KEY, maskParams);
        editor.apply();
    }
    public boolean Load(AsReaderP3xU reader) {
        SharedPreferences sharedPreferences = _context.getSharedPreferences(com.asreader.p3xu.MainActivity.APP_NAME, Context.MODE_PRIVATE);
        String maskParams = sharedPreferences.getString(KEY, ""); // 从缓存里读
        if(maskParams.isEmpty()) {
            _configParams = new ConfigParams();
        } else {
            _configParams = _gson.fromJson(maskParams, ConfigParams.class);
        }

        return true;
    }
    public void resetContents() {
        _configParams._epcMasks.clear();
        _configParams._session = AsReaderP3xUQuerySession.S0;
        // _configParams._target = QueryTarget.AB;  // zsj-2020-12-28 - Option界面注释
        _configParams._target = AsReaderP3xUSessionFlag.A;  // zsj-2020-12-28 - 默认值
        _configParams._reportRssi = false;
        _configParams._displayPc = true;
        _configParams._inventoryTime = 100;
        _configParams._idleTime = 400;
        _configParams._hardwareVersion = "";
    }

    public AsReaderP3xUQuerySession getSession() {
        return _configParams._session;
    }
    public void setSession(AsReaderP3xUQuerySession value) {
        _configParams._session = value;
    }
    public AsReaderP3xUSessionFlag getTarget() {
        return _configParams._target;
    }
    public void setTarget(AsReaderP3xUSessionFlag value) {
        _configParams._target = value;
    }

    public void clearEpcMasks() {
        _configParams._epcMasks.clear();
    }

    public void addEpcMask(AsReaderP3xUSelectMaskEpcParam value) {
        _configParams._epcMasks.add(value);
    }
    public AsReaderP3xUSelectMaskEpcParam getEpcMask(int index) {
        return _configParams._epcMasks.get(index);
    }

    public int getEpcMaskCount() {
        return _configParams._epcMasks.size();
    }

    public boolean getReportRssi() {
        return _configParams._reportRssi;
    }
    public void setReportRssi(boolean value) {
        _configParams._reportRssi = value;
    }

    public boolean getDisplayPc() {
        return _configParams._displayPc;
    }
    public void setDisplayPc(boolean value) {
        _configParams._displayPc = value;
    }

    public int getPowerGain() {
        return _configParams._powerGain;
    }
    public void setPowerGain(int value) {
        _configParams._powerGain = value;
    }

    public int getOperationTime() {
        return _configParams._operationTime;
    }
    public void setOperationTime(int value) {
        _configParams._operationTime = value;
    }

    public int getAutoOffTime() {
        return _configParams._autoOffTime;
    }
    public void setBatteryInterval(int value) {
        _configParams._batteryInterval = value;
    }

    public int getBatteryInterval() {
        return _configParams._batteryInterval;
    }
    public void setAutoOffTime(int value) {
        _configParams._autoOffTime = value;
    }

    public int getQValue() {
        return _configParams._qValue;
    }
    public void setQValue(int value) {
        _configParams._qValue = value;
    }

    public int getInventoryTime() {
        return _configParams._inventoryTime;
    }

    public void setInventoryTime(int value) {
        _configParams._inventoryTime = value;
    }

    public int getIdleTime() {
        return _configParams._idleTime;
    }

    public void setIdleTime(int value) {
        _configParams._idleTime = value;
    }

    public String getHardwareVersion() {
        return _configParams._hardwareVersion;
    }

    public void setHardwareVersion(String value) {
        _configParams._hardwareVersion = value;
    }
    public AsReaderP3xUBuzzerState getVolume() { return _configParams._volume; }
    public void setVolume (AsReaderP3xUBuzzerState value) { _configParams._volume = value; }

    public AsReaderP3xUVibratorState getVibrator() {
        return _configParams._vibrator;
    }
    public void setVibrator(AsReaderP3xUVibratorState value) {
        _configParams._vibrator = value;
    }

    public boolean getContinuousMode() {
        return _configParams._continuousMode;
    }
    public void setContinuousMode(boolean value) {
        _configParams._continuousMode = value;
    }

    public boolean getFrequencyAuto() {
        return _configParams._frequencyAuto;
    }
    public void setFrequencyAuto(boolean value) {
        _configParams._frequencyAuto = value;
    }

    public String getFrequency() {
        return _configParams._frequency;
    }
    public void setFrequency(String value) {
        _configParams._frequency = value;
    }

    private class ConfigParams {
        private ArrayList<AsReaderP3xUSelectMaskEpcParam> _epcMasks = null;
        private AsReaderP3xUQuerySession _session = AsReaderP3xUQuerySession.S0;
        private AsReaderP3xUSessionFlag _target = AsReaderP3xUSessionFlag.A;
        private boolean _reportRssi = false;
        private boolean _displayPc = true;
        private int _powerGain = 0;
        private int _operationTime = 0;
        private int _batteryInterval = 0;
        private int _inventoryTime = 400;
        private int _idleTime = 50;
        private int _autoOffTime = 0;
        private int _qValue = 0;
        private AsReaderP3xUBuzzerState _volume = AsReaderP3xUBuzzerState.High;
        private AsReaderP3xUVibratorState _vibrator = AsReaderP3xUVibratorState.On;
        private boolean _continuousMode = true;
        private boolean _frequencyAuto = true;
        private String _frequency = "";
        private String _hardwareVersion = "";

        private ConfigParams() {
            _volume = AsReaderP3xUBuzzerState.High;
            _vibrator = AsReaderP3xUVibratorState.On;
            _autoOffTime = 1800;
            _qValue = 4;
            _operationTime = 0;
            _powerGain = 260;
            _session = AsReaderP3xUQuerySession.S0;
            _continuousMode = true;
            _reportRssi = false;
            _displayPc = true;
            _target = AsReaderP3xUSessionFlag.AB;
            _inventoryTime = 100;
            _idleTime = 400;
            _epcMasks = new ArrayList<AsReaderP3xUSelectMaskEpcParam>();
            _batteryInterval = 10;
            _hardwareVersion = "";
        }
    }
}
