package com.asreader.support.andro.app.rfid.view.base;

import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;
import com.asreader.R;
import com.asreader.support.andro.app.rfid.util.ResUtil;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

public abstract class AccessActivity extends RfidMaskActivity implements OnCheckedChangeListener {

	private static final String TAG = AccessActivity.class.getSimpleName();

	public static final int VIEW_READ_MEMORY = 1003;
	public static final int VIEW_WRITE_MEMORY = 1004;
	public static final int VIEW_TAG_ACCESS = 1005;

	private TextView txtSelection;
	private ProgressBar prgWait;
	private TextView txtMessage;
	private LinearLayout layoutMessage;
	private TextView txtPassword;
	private TextView txtPowerGain;
	private TextView txtOperationTime;

	//private boolean mLastContinuousMode;
	private String mMask;
	private String mPassword;

	private boolean mIsActionResult;

	public AccessActivity() {
		mMask = "";
		mIsActionResult = false;
	}

	@Override
	protected void initAsReaderP3xU() {
		super.initAsReaderP3xU();

		try {
			getAsReaderP3xU().setContinuousMode(false);
		} catch (AsReaderP3xUException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to set continuous mode");
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initReader()");
	}

	@Override
	protected void destroyAsReaderP3xU() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. destroyAsReaderP3xU()");
		super.destroyAsReaderP3xU();
	}

	@Override
	public void onClick(View v) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.password:
			showAccessPasswordDialog();
			break;
		}
		
		super.onClick(v);
	}

	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCheckedChanged(%s, %s)", ResUtil.getId(buttonView.getId()), isChecked);
	}

	// Stop Action
	protected void stopAction() {
		AsReaderP3xUResultCode res;
		enableActionWidgets(false);
		outputMessage(R.string.stop_action_message);

		if ((res = getAsReaderP3xU().stop()) != AsReaderP3xUResultCode.NoError) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. stopAction() - Failed to stop operation [%s]", res);
			enableActionWidgets(true);
			outputFailMessage(R.string.stop_action_fail_message);
			return;
		}

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. stopAction()");
	}

	@Override
	protected void initWidgets() {
		super.initWidgets();

		// Selection TextView
		txtSelection = (TextView) findViewById(R.id.selection);
		// Wait ProgressBar
		prgWait = (ProgressBar) findViewById(R.id.progress_bar);
		// Message TextView
		txtMessage = (TextView) findViewById(R.id.message);
		// Message Background
		layoutMessage = (LinearLayout) findViewById(R.id.background);

		// Password EditText
		txtPassword = (TextView) findViewById(R.id.password);
		txtPassword.setOnClickListener(this);
		setAccessPassword("");
		
		txtPowerGain = (TextView) findViewById(R.id.power_gain);
		txtPowerGain.setOnClickListener(this);
		txtOperationTime = (TextView) findViewById(R.id.operation_time);
		txtOperationTime.setOnClickListener(this);

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);
		prgWait.setVisibility(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop ? View.GONE : View.VISIBLE);
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	@Override
	protected void clearWidgets() {
		outputSelection(" ");
		outputMessage(" ");

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. clearWidgets()");
	}

	protected void setMask(String mask) {
		mMask = mask;
		txtSelection.setText(mask);

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setMask([%s])", mask);
	}

	protected void outputSelection(String epc) {
		if (!mMask.equals(""))
			return;

		if(epc.length() > 36) {
			epc = epc.substring(0, 36);
			epc += "...";
		}

		txtSelection.setText(epc);
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. outputSelection([%s])", epc);
	}

	protected void outputMessage(int id) {
		outputMessage(getString(id));
	}

	protected void outputMessage(String msg) {
		txtMessage.setText(msg);
		txtMessage.setTextColor(getColor(getApplicationContext(), R.color.black));
		layoutMessage.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. outputMessage([%s])", msg);
	}

	protected void outputSuccessMessage(int id) {
		outputSuccessMessage(getString(id));
	}

	protected void outputSuccessMessage(String msg) {
		prgWait.setVisibility(View.GONE);
		txtMessage.setText(msg);
		txtMessage.setTextColor(getColor(getApplicationContext(), R.color.black));
		layoutMessage.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. outputSuccessMessage([%s])", msg);
	}

	protected void outputFailMessage(int id) {
		outputFailMessage(getString(id));
	}

	protected void outputFailMessage(String msg) {
		prgWait.setVisibility(View.GONE);
		txtMessage.setText(msg);
		txtMessage.setTextColor(getColor(getApplicationContext(), R.color.white));
		layoutMessage.setBackgroundColor(getColor(getApplicationContext(), R.color.red));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. outputFailMessage([%s])", msg);
	}

	protected synchronized boolean isActionResult() {
		return mIsActionResult;
	}

	protected synchronized void setActionResult() {
		mIsActionResult = true;
	}

	protected synchronized void resetActionResult() {
		mIsActionResult = false;
	}

	protected void setAccessPassword(String password) {
		mPassword = password;
		txtPassword.setText(mPassword);
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setAccessPassword([%s])", password);
	}

	protected String getAccessPassword() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. getAccessPassword() - [%s]", mPassword);
		return mPassword;
	}

	protected String getRequestCode(int requestCode) {
		switch (requestCode) {
		case VIEW_READ_MEMORY:
			return "VIEW_READ_MEMORY";
		case VIEW_WRITE_MEMORY:
			return "VIEW_WRITE_MEMORY";
		case VIEW_TAG_ACCESS:
			return "VIEW_TAG_ACCESS";
		}
		return super.getRequestCode(requestCode);
	}

	private void showAccessPasswordDialog() {
		if (!mIsEnabled)
			return;
		
		LinearLayout root = (LinearLayout)LinearLayout.inflate(this, R.layout.dialog_password, null);
		final EditText password = (EditText)root.findViewById(R.id.password);
		
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.password);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				setAccessPassword(password.getText().toString());
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(password.getWindowToken(), 0);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAccessPasswordDialog().$PositiveButton.onClick()");
				}
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(password.getWindowToken(), 0);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAccessPasswordDialog().$NegativeButton.onClick()");
				}
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
			
			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(password.getWindowToken(), 0);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAccessPasswordDialog().onCancel()");
				}
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {
			
			@Override
			public void onShow(DialogInterface dialog) {
				password.setText(mPassword);
				password.selectAll();
				password.requestFocus();
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.showSoftInput(password, InputMethodManager.SHOW_FORCED);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAccessPasswordDialog().onShow()");
				}
			}
		});
		dialog.show();
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAccessPasswordDialog()");
	}
}
