package com.asreader.support.andro.app.rfid.view.base;

import java.util.ArrayList;
import java.util.Locale;

import com.asreader.p3xu.MainActivity;
import com.asreader.p3xu.rfid.params.AsReaderP3xUSelectMaskEpcParam;
import com.asreader.p3xu.rfid.type.AsReaderP3xUCommandType;
import com.asreader.p3xu.view.ReadMemoryActivity;
import com.asreader.p3xu.view.TagAccessActivity;
import com.asreader.p3xu.view.WriteMemoryActivity;
import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;
import com.asreader.R;
import com.asreader.support.andro.app.rfid.adapter.TagListAdapter;
import com.asreader.support.andro.app.rfid.data.MySQLiteOpenHelper;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.util.ResUtil;

import android.content.Intent;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public abstract class BaseInventoryActivity extends RfidMaskActivity implements OnCheckedChangeListener {

	private static final String TAG = BaseInventoryActivity.class.getSimpleName();

	protected ListView lstTags;
	protected TextView txtCount;
	protected Button btnInventory;

	protected TagListAdapter adpTags;

	private MenuItem mnuReadMemory;
	private MenuItem mnuWriteMemory;
	private MenuItem mnuTagAccess;
	
	private boolean mLastContinuousMode = true;
	private boolean mLastReportRssi = false;

	// DB // zsj 2021-02-26 (from 251G)
//	private TextView txtTotalCount;
	private long mTotalTagCount;
	protected MySQLiteOpenHelper dbHelper;

	@Override
	protected void onStart() {
		super.onStart();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onResume()");
//		try {
//			getAsReaderP3xU().clearEpcMask();
//		} catch (AsReaderP3xUException e) {
//			e.printStackTrace();
//		}
		try {
			if(asReaderP3xUSelectMaskEpcParam.getMask().length() != 0) {
				getAsReaderP3xU().addEpcMask(asReaderP3xUSelectMaskEpcParam);
			}
		} catch (AsReaderP3xUException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected void onPause() {
		super.onPause();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onPause()");
	}

	@Override
	protected void onStop() {
		super.onStop();
		insertDB(); // DB // zsj 2021-02-26 （code from 251G)）
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStop()");
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		if (resultCode == RESULT_DISCONNECTED) {
			setResult(RESULT_DISCONNECTED);
			finish();
			return;
		}

		switch (requestCode) {
		case AccessActivity.VIEW_READ_MEMORY:
		case AccessActivity.VIEW_WRITE_MEMORY:
		case AccessActivity.VIEW_TAG_ACCESS:
			try {
				getAsReaderP3xU().setContinuousMode(mLastContinuousMode);
			} catch (AsReaderP3xUException e) {
				AsReaderP3xUASRLog.e(TAG, "onActivityResult - Failed to setContinuousMode");
			}
			try {
				getAsReaderP3xU().setReportRSSI(mLastReportRssi);
				MainActivity.mUserConfigs.setReportRssi(mLastReportRssi);
				MainActivity.mUserConfigs.Save();
			} catch (AsReaderP3xUException e) {
				AsReaderP3xUASRLog.e(TAG, "onActivityResult - Failed to setReportRSSI");
			}
			enableActionWidgets(true);
			return;
		}
		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreateContextMenu()");

		if (v.getId() != R.id.tag_list || getAsReaderP3xU().getAction() != AsReaderP3xUActionState.Stop)
			return;

		getMenuInflater().inflate(R.menu.inventory_menu, menu);

		mnuReadMemory = menu.findItem(R.id.read_memory);
		mnuWriteMemory = menu.findItem(R.id.write_memory);
		mnuTagAccess = menu.findItem(R.id.tag_access);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onContextItemSelected(%s)", ResUtil.getId(item.getItemId()));

		Intent intent;
		AdapterView.AdapterContextMenuInfo menuInfo = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
		int position = menuInfo.position;

		if (position < 0)
			return false;

		mnuReadMemory.setEnabled(false);
		mnuWriteMemory.setEnabled(false);
		mnuTagAccess.setEnabled(false);

		String tag = adpTags.getItem(position, false);
		
		try {
			mLastContinuousMode = getAsReaderP3xU().getContinuousMode();
		} catch (AsReaderP3xUException e) {
			AsReaderP3xUASRLog.e(TAG, "onContextItemSelected - Failed to getContinuousMode");
		}
		
		try {
			mLastReportRssi = getAsReaderP3xU().getReportRSSI();
		} catch (AsReaderP3xUException e) {
			AsReaderP3xUASRLog.e(TAG, "onContextItemSelected - Failed to getReportRSSI");
		}
		
		switch (item.getItemId()) {
		case R.id.read_memory:
			intent = new Intent(this, ReadMemoryActivity.class);
			intent.putExtra(KEY_MASK, tag);
			setupMask(tag);
			startActivityForResult(intent, AccessActivity.VIEW_READ_MEMORY);
			break;
		case R.id.write_memory:
			intent = new Intent(this, WriteMemoryActivity.class);
			intent.putExtra(KEY_MASK, tag);
			setupMask(tag);
			startActivityForResult(intent, AccessActivity.VIEW_WRITE_MEMORY);
			break;
		case R.id.tag_access:
			intent = new Intent(this, TagAccessActivity.class);
			intent.putExtra(KEY_MASK, tag);
			setupMask(tag);
			startActivityForResult(intent, AccessActivity.VIEW_TAG_ACCESS);
			break;
		}
		return true;
	}
	private void setupMask(String tag) {
		try {
			int count = getAsReaderP3xU().getEpcMaskCount();
			if (count != 0) {
				asReaderP3xUSelectMaskEpcParam = getAsReaderP3xU().getEpcMask(0);
				if (asReaderP3xUSelectMaskEpcParam == null) {
					asReaderP3xUSelectMaskEpcParam = new AsReaderP3xUSelectMaskEpcParam();
					asReaderP3xUSelectMaskEpcParam.setOffset(16);
					asReaderP3xUSelectMaskEpcParam.setLength(16);
				}
			} else {
				asReaderP3xUSelectMaskEpcParam = new AsReaderP3xUSelectMaskEpcParam();
				asReaderP3xUSelectMaskEpcParam.setOffset(16);
				asReaderP3xUSelectMaskEpcParam.setLength(16);
			}
		} catch (AsReaderP3xUException e) {
			asReaderP3xUSelectMaskEpcParam = new AsReaderP3xUSelectMaskEpcParam();
			asReaderP3xUSelectMaskEpcParam.setOffset(16);
			asReaderP3xUSelectMaskEpcParam.setLength(16);
			e.printStackTrace();
		}
		AsReaderP3xUSelectMaskEpcParam param = new AsReaderP3xUSelectMaskEpcParam();
		param.setOffset(32);
		param.setMask(tag);
		param.setLength(tag.length() * 4);
		try {
			getAsReaderP3xU().clearEpcMask();
		} catch (AsReaderP3xUException e) {
			e.printStackTrace();
		}
		try {
			getAsReaderP3xU().addEpcMask(param);
		} catch (AsReaderP3xUException e) {
			e.printStackTrace();
		}
	}
	public AsReaderP3xUSelectMaskEpcParam asReaderP3xUSelectMaskEpcParam = new AsReaderP3xUSelectMaskEpcParam();

	@Override
	public void onClick(View v) {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.inventory:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				startInventory();
			} else {
				stopInventory();
			}
			break;
		}

		super.onClick(v);
	}

	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCheckedChanged(%s, %s)", ResUtil.getId(buttonView.getId()), isChecked);
	}

	@Override
	public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onActionChanged(%s)", action);

//		adpTags.notifyDataSetChanged();

		enableActionWidgets(true);
	}

	@Override
	public void onReadTag(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action, String tag, float rssi) {

		mTotalTagCount++;
		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "EVENT. onReadTag(%s, [%s], %.2f)", action, tag, rssi);

//		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));
		adpTags.addTag(tag, rssi, System.currentTimeMillis());
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));

		adpTags.notifyDataSetChanged();
	}
	
	@Override
	public void onAccessResult(AsReaderP3xU asReaderP3xU, AsReaderP3xUResultCode code, AsReaderP3xUActionState action, String epc, String data) {
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onAccessResult(%s, [%s], [%s])", action, epc, data);
	}

	// Initialize Widgets
	@Override
	protected void initWidgets() {
		super.initWidgets();

		// Total Count TextView // for DB // zsj 2021-02-26 （code from 251G)）
//		txtTotalCount = (TextView) findViewById(R.id.total_tag_count);
//		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));
		mTotalTagCount = 0;

		btnOption = (Button) findViewById(R.id.option);
		if(btnOption != null)
			btnOption.setOnClickListener(this);

		// Tag ListView
		lstTags = (ListView) findViewById(R.id.tag_list);
		adpTags = new TagListAdapter(this);

		// new database
		dbHelper = MySQLiteOpenHelper.getInstance(this); // DB // zsj 2021-02-26 （code from 251G)）

		// Count TextView
		txtCount = (TextView) findViewById(R.id.tag_count);
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));
		// Inventory Button
		btnInventory = (Button) findViewById(R.id.inventory);
		btnInventory.setOnClickListener(this);

		getDB(); // DB // zsj 2021-02-26 （code from 251G)）

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);
		btnMask.setEnabled(isEnabledWidget(enabled));

//		tvDisplayPc.setEnabled(enabled);
//		tvContinuousMode.setEnabled(enabled);

		lstTags.setEnabled(isEnabledWidget(enabled));

		enableActionButton(btnInventory, enabled, R.string.action_inventory, R.string.action_stop);
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}
	
	@Override
	protected void clearWidgets() {
		mTotalTagCount = 0;
		adpTags.clear();
//		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. clearWidgets()");
	}

	protected void startInventory() {
		enableActionWidgets(false);
		AsReaderP3xUResultCode res;
		if ((res = getAsReaderP3xU().inventory()) != AsReaderP3xUResultCode.NoError) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. startInventory() - Failed to start inventory()");
			String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
			Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
			enableActionWidgets(true);
			return;
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. startInventory()");
	}

	protected void stopInventory() {
		enableActionWidgets(false);
		if (getAsReaderP3xU().stop() != AsReaderP3xUResultCode.NoError) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. stopInventory() - Failed to stop inventory()");
			enableActionWidgets(true);
			insertDB(); // DB // zsj 2021-02-26 （code from 251G)）
			return;
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. stopInventory()");
	}

	public void setReportRssiMode(boolean enabled) {
		// 1) Save current data list to DB.
		insertDB(); // DB // zsj 2021-02-26 （code from 251G)）

		this.adpTags.setReportRSSI(enabled);

		// 2) on the list of currently cleared get DB data.
		getDB(); // DB // zsj 2021-02-26 （code from 251G)）
	}
	
	public void setDisplayPC(boolean enabled) {
		this.adpTags.setDisplayPc(enabled);
	}

	// zsj 2021-02-26 （code from 251G)）
	protected void getDB() {
		// data from db
		ArrayList dbTag = dbHelper.select();

		for (int i=0; i<dbTag.size(); i++){
			if (dbTag.get(i).equals(""))
				break;

			String[] temp = dbTag.get(i).toString().split(",");
//			Log.d(TAG, "# DB get : "+ dbTag.get(i));

			adpTags.addDBTag(temp[0], Integer.parseInt(temp[1]),Float.parseFloat(temp[2]),Long.parseLong(temp[3]));
			mTotalTagCount = Long.parseLong(temp[4]);
		}

		lstTags.setAdapter(adpTags);
		adpTags.start();

		registerForContextMenu(lstTags);

		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));
//		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));
	}

	protected void insertDB() {
		// DB data delete.
		dbHelper.delete();

		// DB insert
		for (int i =0; i<adpTags.getCount(); i++){
//			Log.d(TAG,"# DB stop : set DB List item : " + adpTags.getItemForDB(i));

			String[] temp = adpTags.getItemForDB(i).split(",");
			dbHelper.insert(temp[0],temp[1],temp[2],mTotalTagCount,temp[3]);
		}
	}
}
