package com.asreader.support.andro.app.rfid.view.base;

import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.AsReaderP3xUManager;
import com.asreader.p3xu.device.type.AsReaderP3xUConnectionState;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.event.AsReaderP3xUEventListener;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyType;
import com.asreader.p3xu.rfid.type.AsReaderP3xUCommandType;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;
import com.asreader.support.andro.app.rfid.util.ResUtil;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.widget.Button;

public abstract class RfidActivity extends SubActivity implements AsReaderP3xUEventListener {

	private static final String TAG = RfidActivity.class.getSimpleName();

	protected static final int VIEW_MASK = 1000;

	protected static final int RESULT_DISCONNECTED = Activity.RESULT_FIRST_USER + 1;

	public static final String KEY_ACTION = "key_action";
	
	public static final boolean DEFAULT_KEY_ACTION = true;

	private AsReaderP3xU mAsReaderP3xU = null;
	private boolean mKeyAction;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	
		mKeyAction = getIntent().getBooleanExtra(KEY_ACTION, DEFAULT_KEY_ACTION);
	}

	// Retreive RFID AsReaderP3xU
	protected AsReaderP3xU getAsReaderP3xU() {
		return mAsReaderP3xU;
	}

	// Create AsReaderP3xU
	protected void createAsReaderP3xU() {
		mAsReaderP3xU = AsReaderP3xUManager.getInstance();
		mAsReaderP3xU.setEventListener(this);
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. createAsReaderP3xU()");
	}

	// Destroy AsReaderP3xU
	protected void destroyAsReaderP3xU() {
		mAsReaderP3xU.removeEventListener(this);

		if (mAsReaderP3xU.getAction() != AsReaderP3xUActionState.Stop &&
				mAsReaderP3xU.getState() == AsReaderP3xUConnectionState.Connected) {
			mAsReaderP3xU.stop();
		}

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. destroyAsReaderP3xU()");
	}

	// Initialize P3xU
	protected abstract void initAsReaderP3xU();

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		//if (requestCode == VIEW_MASK && resultCode == RESULT_DISCONNECTED) {
		if (requestCode == VIEW_MASK && resultCode == RESULT_DISCONNECTED) {
			setResult(RESULT_DISCONNECTED);
			finish();
			return;
		}

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public void onStateChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUConnectionState state) {
		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "EVENT. onStateChanged(%s)", state);

		if (state == AsReaderP3xUConnectionState.Disconnected) {
			setResult(RESULT_DISCONNECTED);
			finish();
		}
	}

	@Override
	public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {
		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "EVENT. onActionChanged(%s)", action);
	}

	@Override
	public void onReadTag(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action, String tag, float rssi) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onReadedTag(%s, [%s], %.2f)", action, tag, rssi);
	}

	@Override
	public void onAccessResult(AsReaderP3xU asReaderP3xU, AsReaderP3xUResultCode code, AsReaderP3xUActionState action, String epc, String data) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s])", code, action, epc, data);
	}

	@Override
	public void onKeyEvent(AsReaderP3xU asReaderP3xU, AsReaderP3xUKeyType key, AsReaderP3xUKeyState state) {
			
	}

	@Override
	protected void initWidgets() {
		
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		
	}

	protected void enableActionButton(Button action, boolean enabled, int actionId, int stopId) {
		action.setEnabled(enabled);
		
		if (enabled) {
			action.setText(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop ? actionId : stopId);
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionButton(%s, %s, %d, %d)", ResUtil.getId(action.getId()), enabled, actionId,
				stopId);
	}

	protected boolean isEnabledWidget(boolean enabled) {
		AsReaderP3xUActionState action = getAsReaderP3xU().getAction();
		return enabled ? (action == AsReaderP3xUActionState.Stop ? enabled : false) : enabled;
	}

	protected boolean isEnabledWidget(boolean enabled, AsReaderP3xUActionState state) {
		AsReaderP3xUActionState action = getAsReaderP3xU().getAction();
		return enabled ? (action == AsReaderP3xUActionState.Stop ? enabled : (action == state ? enabled : false)) : enabled;
	}

	protected String getResultCode(int resultCode) {

		switch (resultCode) {
		case RESULT_DISCONNECTED:
			return "RESULT_DISCONNECTED";
		}
		return super.getResultCode(resultCode);
	}
	
	protected boolean getKeyAction() {
		return mKeyAction;
	}
	
	protected void setKeyAction(boolean enabled) {
		mKeyAction = enabled;
	}
	
	@SuppressLint("NewApi")
	@SuppressWarnings("deprecation")
	protected int getColor(Context context, int id) {
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			return context.getColor(id);
		else
			return context.getResources().getColor(id);
	}
}
