package com.asreader.support.andro.app.rfid.view.base;

import java.util.Locale;

import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.params.AsReaderP3xUPowerRange;
import com.asreader.R;
import com.asreader.support.andro.app.rfid.adapter.PowerRangeAdapter;
import com.asreader.support.andro.app.rfid.filter.InputFilterMinMax;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.util.UserConfigs;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.text.InputFilter;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public abstract class RfidPowerGainActivity extends RfidClearActivity {

	private static final String TAG = RfidPowerGainActivity.class.getSimpleName();

	private static final int DEFAULT_POWER_GAIN = 260;

	private TextView txtPower;
	private TextView txtOperationTime;

	private int mPowerGain;
	private int mOperationTime;
	private AsReaderP3xUPowerRange mPowerRange;

	protected boolean mIsEnabled;

	@Override
	protected void createAsReaderP3xU() {
		super.createAsReaderP3xU();
		mPowerGain = DEFAULT_POWER_GAIN;

		// zsj-2020-12-28 - Memory界面注释
		try {
			mPowerRange = getAsReaderP3xU().getPowerGainRange();
		} catch (Exception e) {
			mPowerRange = new AsReaderP3xUPowerRange(150, DEFAULT_POWER_GAIN);
		}

		mIsEnabled = false;

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. createReader()");
	}

	// Initialize Reader
	@Override
	protected void initAsReaderP3xU() {
		if(txtPower != null) {
			// Get Power Gain
			int power;
			try {
				power = getAsReaderP3xU().getPowerGain(); // write:7e72700d0a  ~rp   没回
			} catch (AsReaderP3xUException e) {
				power = 0;
			}
			setPowerGain(power);
		}

		if (txtOperationTime != null) {
			int time;
			// Get Operation Time
			try {
				time = getAsReaderP3xU().getOperationTime(); // write:7e72690d0a  回 7e526930303030  303030303030464b30303131  0000... // ~Ri0000
			} catch (AsReaderP3xUException e) {
				AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get operation time");
				time = 0;
			}
			setOperationTime(time);
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initReader()");
	}

	@Override
	public void onClick(View v) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.power_gain:
			showPowerGainDialog();
			break;
		case R.id.operation_time:
			showOperationTimeDialog();
			break;
		}
		super.onClick(v);
	}

	protected void initWidgets() {
		super.initWidgets();

		// Power Gain Spinner
		txtPower = (TextView) findViewById(R.id.power_gain);
		if(txtPower != null)
			txtPower.setOnClickListener(this);
		// Operation Time EditText
		txtOperationTime = (TextView) findViewById(R.id.operation_time);
		if(txtOperationTime != null)
			txtOperationTime.setOnClickListener(this);
		setOperationTime(mOperationTime);
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		mIsEnabled = isEnabledWidget(enabled);

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	protected void setPowerGain(int power) {
		
		if(txtPower == null)
			return;
		
		mPowerGain = power;
		txtPower.setText(String.format(Locale.US, "%.1f dBm", (double) power / 10.0));

		try {
			getAsReaderP3xU().setPowerGain(mPowerGain);
			UserConfigs.getInstance(this.getApplicationContext()).setPowerGain(mPowerGain);
			UserConfigs.getInstance(this.getApplicationContext()).Save();
		} catch (AsReaderP3xUException e) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. setPowerGain(%d) - Failed to set power gain", mPowerGain);
			return;
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setPowerGain(%d)", power);
	}

	@SuppressWarnings("unused")
	protected int getPowerGain() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. getPowerGain() - [%d]", mPowerGain);
		return mPowerGain;
	}

	protected void setOperationTime(int time) {
		if(txtOperationTime == null)
			return;
		
		mOperationTime = time;
		txtOperationTime.setText(String.format(Locale.US, "%d s", mOperationTime));
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setOperationTime(%d)", time);
	}

	protected int getOperationTime() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. getOperationTime() - [%d]", mOperationTime);
		return mOperationTime;
	}




	private void showPowerGainDialog() {

		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView power = (ListView) root.findViewById(R.id.list);
		final PowerRangeAdapter adapter = new PowerRangeAdapter(this, mPowerRange,
				android.R.layout.simple_list_item_single_choice);
		power.setAdapter(adapter);
		power.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.power_gain);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = power.getCheckedItemPosition();
				int power = adapter.getItemValue(position);
				setPowerGain(power);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
			
			@Override
			public void onCancel(DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getItemPosition(mPowerGain);
				power.setItemChecked(position, true);
				power.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().onShow()");
			}
		});
		dialog.show();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog()");
	}

	private void showOperationTimeDialog() {

		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final TextView unit = (TextView) root.findViewById(R.id.unit);
		unit.setText(R.string.unit_sec);

		final EditText operationTime = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(0, 1800) };
		operationTime.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.operation_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int time;
				try {
					time = Integer.parseInt(operationTime.getText().toString());
				} catch (Exception e) {
					time = 0;
				}
				try {
					getAsReaderP3xU().setOperationTime(time);
					UserConfigs.getInstance(RfidPowerGainActivity.this.getApplicationContext()).setOperationTime(time);
					UserConfigs.getInstance(RfidPowerGainActivity.this.getApplicationContext()).Save();
				} catch (AsReaderP3xUException e) {
					AsReaderP3xUASRLog.e(TAG, e, "ERROR. showOperationTimeDialog() - Failed to set operation time(%d)", time);
				}
				setOperationTime(time);
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().$PositiveButton.onClick()");
				}
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().$NegativeButton.onClick()");
				}
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().onCancel()");
				}
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				operationTime.setText(String.format(Locale.US, "%d", mOperationTime));
				operationTime.selectAll();
				operationTime.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.showSoftInput(operationTime, InputMethodManager.SHOW_FORCED);
					AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().onShow()");
				}
			}
		});
		dialog.show();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog()");
	}
}
